package com.compomics.marc;

import com.compomics.util.math.BasicMathFunctions;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import org.apache.commons.math.MathException;
import org.apache.commons.math.util.FastMath;

/**
 *
 * @author Marc Vaudel
 */
public class JillMSLF {

    private static final String separator = "\t";

    public static void main(String[] args) {

        JillMSLF m = new JillMSLF();

        try {
            m.jillRoc();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void jillRoc() throws FileNotFoundException, IOException, MathException {

        // Input
        File workingFolder = new File("D:\\projects\\jill roc\\");
        File patientMappingFile = new File(workingFolder, "patient mapping.txt");

        HashMap<String, String> patientNameToCategory = new HashMap<String, String>();
        HashMap<String, String> patientKeyToCategory = new HashMap<String, String>();
        HashMap<String, String> patientNameToKey = new HashMap<String, String>();
        loadPatientMapping(patientMappingFile, patientNameToCategory, patientKeyToCategory, patientNameToKey);

        File inputUnaffected = new File(workingFolder, "blood unaffected.txt");
        File inputUncertain = new File(workingFolder, "blood affect uncertain.txt");
        File inputAffected = new File(workingFolder, "blood affected.txt");
        HashMap<String, Integer> proteinAccessionToCategory = loadProteinMapping(inputUnaffected, inputUncertain, inputAffected);

        File concentrationRawFile = new File(workingFolder, "concentration raw.txt");
        File concentrationNormalizedFile = new File(workingFolder, "concentration normalized.txt");
        File volumeRawFile = new File(workingFolder, "volume raw.txt");
        File volumeNormalizedFile = new File(workingFolder, "volume normalized.txt");

        // Accession -> Patient -> Intensity
        HashMap<String, HashMap<String, Double>> controlInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                ondInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                sasInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                rrmsInputConcentrationRaw = new HashMap<String, HashMap<String, Double>>(),
                controlInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                ondInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                sasInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                rrmsInputVolumeRaw = new HashMap<String, HashMap<String, Double>>(),
                controlInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                ondInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                sasInputConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                rrmsConcentrationNormalized = new HashMap<String, HashMap<String, Double>>(),
                controlInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                ondInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                sasInputVolumeNormalized = new HashMap<String, HashMap<String, Double>>(),
                rrmsVolumeNormalized = new HashMap<String, HashMap<String, Double>>();

        // Load data
        jillLoadIntensities(concentrationRawFile, patientNameToCategory, patientNameToKey, controlInputConcentrationRaw, sasInputConcentrationRaw, ondInputConcentrationRaw, rrmsInputConcentrationRaw);
        jillLoadIntensities(concentrationNormalizedFile, patientNameToCategory, patientNameToKey, controlInputConcentrationNormalized, sasInputConcentrationNormalized, ondInputConcentrationNormalized, rrmsConcentrationNormalized);
        jillLoadIntensities(volumeRawFile, patientKeyToCategory, null, controlInputVolumeRaw, sasInputVolumeRaw, ondInputVolumeRaw, rrmsInputVolumeRaw);
        jillLoadIntensities(volumeNormalizedFile, patientKeyToCategory, null, controlInputVolumeNormalized, sasInputVolumeNormalized, ondInputVolumeNormalized, rrmsVolumeNormalized);

        // Patient -> Mean all
        HashMap<String, Double> controlConcentrationMeanAll = getMeanAll(controlInputConcentrationRaw),
                ondConcentrationMeanAll = getMeanAll(ondInputConcentrationRaw),
                sasConcentrationMeanAll = getMeanAll(sasInputConcentrationRaw),
                rrmsConcentrationMeanAll = getMeanAll(rrmsInputConcentrationRaw),
                controlVolumeMeanAll = getMeanAll(controlInputVolumeRaw),
                ondVolumeMeanAll = getMeanAll(ondInputVolumeRaw),
                sasVolumeMeanAll = getMeanAll(sasInputVolumeRaw),
                rrmsVolumeMeanAll = getMeanAll(rrmsInputVolumeRaw);

        // Patient -> Median all
        HashMap<String, Double> controlConcentrationMedianAll = getMedianAll(controlInputConcentrationRaw),
                ondConcentrationMedianAll = getMedianAll(ondInputConcentrationRaw),
                sasConcentrationMedianAll = getMedianAll(sasInputConcentrationRaw),
                rrmsConcentrationMedianAll = getMedianAll(rrmsInputConcentrationRaw),
                controlVolumeMedianAll = getMedianAll(controlInputVolumeRaw),
                ondVolumeMedianAll = getMedianAll(ondInputVolumeRaw),
                sasVolumeMedianAll = getMedianAll(sasInputVolumeRaw),
                rrmsVolumeMedianAll = getMedianAll(rrmsInputVolumeRaw);

        // Patient -> Median brain
        HashMap<String, Double> controlConcentrationMedianBrain = getMedianBrain(controlInputConcentrationRaw, proteinAccessionToCategory),
                ondConcentrationMedianBrain = getMedianBrain(ondInputConcentrationRaw, proteinAccessionToCategory),
                sasConcentrationMedianBrain = getMedianBrain(sasInputConcentrationRaw, proteinAccessionToCategory),
                rrmsConcentrationMedianBrain = getMedianBrain(rrmsInputConcentrationRaw, proteinAccessionToCategory),
                controlVolumeMedianBrain = getMedianBrain(controlInputVolumeRaw, proteinAccessionToCategory),
                ondVolumeMedianBrain = getMedianBrain(ondInputVolumeRaw, proteinAccessionToCategory),
                sasVolumeMedianBrain = getMedianBrain(sasInputVolumeRaw, proteinAccessionToCategory),
                rrmsVolumeMedianBrain = getMedianBrain(rrmsInputVolumeRaw, proteinAccessionToCategory);

        // Accession -> Patient -> normalized value
        HashMap<String, HashMap<String, Double>> controlConcentrationNormalizedMean = normalizeAll(controlInputConcentrationRaw, controlConcentrationMeanAll),
                ondConcentrationNormalizedMean = normalizeAll(ondInputConcentrationRaw, ondConcentrationMeanAll),
                sasConcentrationNormalizedMean = normalizeAll(sasInputConcentrationRaw, sasConcentrationMeanAll),
                rrmsConcentrationNormalizedMean = normalizeAll(rrmsInputConcentrationRaw, rrmsConcentrationMeanAll),
                controlVolumeNormalizedMean = normalizeAll(controlInputVolumeRaw, controlVolumeMeanAll),
                ondVolumeNormalizedMean = normalizeAll(ondInputVolumeRaw, ondVolumeMeanAll),
                sasVolumeNormalizedMean = normalizeAll(sasInputVolumeRaw, sasVolumeMeanAll),
                rrmsVolumeNormalizedMean = normalizeAll(rrmsInputVolumeRaw, rrmsVolumeMeanAll);

        // Accession -> Patient -> normalized value
        HashMap<String, HashMap<String, Double>> controlConcentrationNormalizedMedian = normalizeAll(controlInputConcentrationRaw, controlConcentrationMedianAll),
                ondConcentrationNormalizedMedian = normalizeAll(ondInputConcentrationRaw, ondConcentrationMedianAll),
                sasConcentrationNormalizedMedian = normalizeAll(sasInputConcentrationRaw, sasConcentrationMedianAll),
                rrmsConcentrationNormalizedMedian = normalizeAll(rrmsInputConcentrationRaw, rrmsConcentrationMedianAll),
                controlVolumeNormalizedMedian = normalizeAll(controlInputVolumeRaw, controlVolumeMedianAll),
                ondVolumeNormalizedMedian = normalizeAll(ondInputVolumeRaw, ondVolumeMedianAll),
                sasVolumeNormalizedMedian = normalizeAll(sasInputVolumeRaw, sasVolumeMedianAll),
                rrmsVolumeNormalizedMedian = normalizeAll(rrmsInputVolumeRaw, rrmsVolumeMedianAll);

        // Accession -> Patient -> normalized value
        HashMap<String, HashMap<String, Double>> controlConcentrationNormalizedBrain = normalizeAll(controlInputConcentrationRaw, controlConcentrationMedianBrain),
                ondConcentrationNormalizedBrain = normalizeAll(ondInputConcentrationRaw, ondConcentrationMedianBrain),
                sasConcentrationNormalizedBrain = normalizeAll(sasInputConcentrationRaw, sasConcentrationMedianBrain),
                rrmsConcentrationNormalizedBrain = normalizeAll(rrmsInputConcentrationRaw, rrmsConcentrationMedianBrain),
                controlVolumeNormalizedBrain = normalizeAll(controlInputVolumeRaw, controlVolumeMedianBrain),
                ondVolumeNormalizedBrain = normalizeAll(ondInputVolumeRaw, ondVolumeMedianBrain),
                sasVolumeNormalizedBrain = normalizeAll(sasInputVolumeRaw, sasVolumeMedianBrain),
                rrmsVolumeNormalizedBrain = normalizeAll(rrmsInputVolumeRaw, rrmsVolumeMedianBrain);

        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlConcentrationRawVariability = getVariability(controlInputConcentrationRaw, proteinAccessionToCategory),
                ondConcentrationRawVariability = getVariability(ondInputConcentrationRaw, proteinAccessionToCategory),
                sasConcentrationRawVariability = getVariability(sasInputConcentrationRaw, proteinAccessionToCategory),
                rrmsConcentrationRawVariability = getVariability(rrmsInputConcentrationRaw, proteinAccessionToCategory),
                controlVolumeRawVariability = getVariability(controlInputVolumeRaw, proteinAccessionToCategory),
                ondVolumeRawVariability = getVariability(ondInputVolumeRaw, proteinAccessionToCategory),
                sasVolumeRawVariability = getVariability(sasInputVolumeRaw, proteinAccessionToCategory),
                rrmsVolumeRawVariability = getVariability(rrmsInputVolumeRaw, proteinAccessionToCategory);

        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlConcentrationMeanVariability = getVariability(controlConcentrationNormalizedMean, proteinAccessionToCategory),
                ondConcentrationMeanVariability = getVariability(ondConcentrationNormalizedMean, proteinAccessionToCategory),
                sasConcentrationMeanVariability = getVariability(sasConcentrationNormalizedMean, proteinAccessionToCategory),
                rrmsConcentrationMeanVariability = getVariability(rrmsConcentrationNormalizedMean, proteinAccessionToCategory),
                controlVolumeMeanVariability = getVariability(controlVolumeNormalizedMean, proteinAccessionToCategory),
                ondVolumeMeanVariability = getVariability(ondVolumeNormalizedMean, proteinAccessionToCategory),
                sasVolumeMeanVariability = getVariability(sasVolumeNormalizedMean, proteinAccessionToCategory),
                rrmsVolumeMeanVariability = getVariability(rrmsVolumeNormalizedMean, proteinAccessionToCategory);

        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlConcentrationMedianVariability = getVariability(controlConcentrationNormalizedMedian, proteinAccessionToCategory),
                ondConcentrationMedianVariability = getVariability(ondConcentrationNormalizedMedian, proteinAccessionToCategory),
                sasConcentrationMedianVariability = getVariability(sasConcentrationNormalizedMedian, proteinAccessionToCategory),
                rrmsConcentrationMedianVariability = getVariability(rrmsConcentrationNormalizedMedian, proteinAccessionToCategory),
                controlVolumeMedianVariability = getVariability(controlVolumeNormalizedMedian, proteinAccessionToCategory),
                ondVolumeMedianVariability = getVariability(ondVolumeNormalizedMedian, proteinAccessionToCategory),
                sasVolumeMedianVariability = getVariability(sasVolumeNormalizedMedian, proteinAccessionToCategory),
                rrmsVolumeMedianVariability = getVariability(rrmsVolumeNormalizedMedian, proteinAccessionToCategory);

        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlConcentrationBrainVariability = getVariability(controlConcentrationNormalizedBrain, proteinAccessionToCategory),
                ondConcentrationBrainVariability = getVariability(ondConcentrationNormalizedBrain, proteinAccessionToCategory),
                sasConcentrationBrainVariability = getVariability(sasConcentrationNormalizedBrain, proteinAccessionToCategory),
                rrmsConcentrationBrainVariability = getVariability(rrmsConcentrationNormalizedBrain, proteinAccessionToCategory),
                controlVolumeBrainVariability = getVariability(controlVolumeNormalizedBrain, proteinAccessionToCategory),
                ondVolumeBrainVariability = getVariability(ondVolumeNormalizedBrain, proteinAccessionToCategory),
                sasVolumeBrainVariability = getVariability(sasVolumeNormalizedBrain, proteinAccessionToCategory),
                rrmsVolumeBrainVariability = getVariability(rrmsVolumeNormalizedBrain, proteinAccessionToCategory);

        // Raw
        File destinationFile = new File("D:\\projects\\jill roc\\variability", "concentratoin_raw_variability.txt");
        writeVariabilityPlot(destinationFile, controlConcentrationRawVariability, ondConcentrationRawVariability, sasConcentrationRawVariability, rrmsConcentrationRawVariability);
        destinationFile = new File("D:\\projects\\jill roc\\variability", "volume_raw_variability.txt");
        writeVariabilityPlot(destinationFile, controlVolumeRawVariability, ondVolumeRawVariability, sasVolumeRawVariability, rrmsVolumeRawVariability);
        // Mean
        destinationFile = new File("D:\\projects\\jill roc\\variability", "concentratoin_mean_variability.txt");
        writeVariabilityPlot(destinationFile, controlConcentrationMeanVariability, ondConcentrationMeanVariability, sasConcentrationMeanVariability, rrmsConcentrationMeanVariability);
        destinationFile = new File("D:\\projects\\jill roc\\variability", "volume_mean_variability.txt");
        writeVariabilityPlot(destinationFile, controlVolumeMeanVariability, ondVolumeMeanVariability, sasVolumeMeanVariability, rrmsVolumeMeanVariability);
        // Mediane
        destinationFile = new File("D:\\projects\\jill roc\\variability", "concentratoin_median_variability.txt");
        writeVariabilityPlot(destinationFile, controlConcentrationMedianVariability, ondConcentrationMedianVariability, sasConcentrationMedianVariability, rrmsConcentrationMedianVariability);
        destinationFile = new File("D:\\projects\\jill roc\\variability", "volume_median_variability.txt");
        writeVariabilityPlot(destinationFile, controlVolumeMedianVariability, ondVolumeMedianVariability, sasVolumeMedianVariability, rrmsVolumeMedianVariability);
        // Brain
        destinationFile = new File("D:\\projects\\jill roc\\variability", "concentratoin_brain_variability.txt");
        writeVariabilityPlot(destinationFile, controlConcentrationBrainVariability, ondConcentrationBrainVariability, sasConcentrationBrainVariability, rrmsConcentrationBrainVariability);
        destinationFile = new File("D:\\projects\\jill roc\\variability", "volume_brain_variability.txt");
        writeVariabilityPlot(destinationFile, controlVolumeBrainVariability, ondVolumeBrainVariability, sasVolumeBrainVariability, rrmsVolumeBrainVariability);

        // BA plots
        
        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlRawBA = getBa(controlInputConcentrationRaw, controlInputVolumeRaw, proteinAccessionToCategory),
                ondRawBA = getBa(ondInputConcentrationRaw, ondInputVolumeRaw, proteinAccessionToCategory),
                sasRawBA = getBa(sasInputConcentrationRaw, sasInputVolumeRaw, proteinAccessionToCategory),
                rrmsRawBA = getBa(rrmsInputConcentrationRaw, rrmsInputVolumeRaw, proteinAccessionToCategory);
        
        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlMeanBA = getBa(controlConcentrationNormalizedMean, controlVolumeNormalizedMean, proteinAccessionToCategory),
                ondMeanBA = getBa(ondConcentrationNormalizedMean, ondVolumeNormalizedMean, proteinAccessionToCategory),
                sasMeanBA = getBa(sasConcentrationNormalizedMean, sasVolumeNormalizedMean, proteinAccessionToCategory),
                rrmsMeanBA = getBa(rrmsConcentrationNormalizedMean, rrmsVolumeNormalizedMean, proteinAccessionToCategory);
        
        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlMedianBA = getBa(controlConcentrationNormalizedMedian, controlVolumeNormalizedMedian, proteinAccessionToCategory),
                ondMedianBA = getBa(ondConcentrationNormalizedMedian, ondVolumeNormalizedMedian, proteinAccessionToCategory),
                sasMedianBA = getBa(sasConcentrationNormalizedMedian, sasVolumeNormalizedMedian, proteinAccessionToCategory),
                rrmsMedianBA = getBa(rrmsConcentrationNormalizedMedian, rrmsVolumeNormalizedMedian, proteinAccessionToCategory);
        
        // Protein category -> Intensity -> variability
        HashMap<Integer, ArrayList<Double>[]> controlBrainBA = getBa(controlConcentrationNormalizedBrain, controlVolumeNormalizedBrain, proteinAccessionToCategory),
                ondBrainBA = getBa(ondConcentrationNormalizedBrain, ondVolumeNormalizedBrain, proteinAccessionToCategory),
                sasBrainBA = getBa(sasConcentrationNormalizedBrain, sasVolumeNormalizedBrain, proteinAccessionToCategory),
                rrmsBrainBA = getBa(rrmsConcentrationNormalizedBrain, rrmsVolumeNormalizedBrain, proteinAccessionToCategory);
        
        // Raw
        destinationFile = new File("D:\\projects\\jill roc\\BA", "raw_ba.txt");
        writeVariabilityPlot(destinationFile, controlRawBA, ondRawBA, sasRawBA, rrmsRawBA);
        // Mean
        destinationFile = new File("D:\\projects\\jill roc\\BA", "mean_ba.txt");
        writeVariabilityPlot(destinationFile, controlMeanBA, ondMeanBA, sasMeanBA, rrmsMeanBA);
        // Mediane
        destinationFile = new File("D:\\projects\\jill roc\\BA", "median_ba.txt");
        writeVariabilityPlot(destinationFile, controlMedianBA, ondMedianBA, sasMedianBA, rrmsMedianBA);
        // Brain
        destinationFile = new File("D:\\projects\\jill roc\\BA", "brain_ba.txt");
        writeVariabilityPlot(destinationFile, controlBrainBA, ondBrainBA, sasBrainBA, rrmsBrainBA);
        

        int end = 1;
    }

    private HashMap<Integer, ArrayList<Double>[]> getBa(HashMap<String, HashMap<String, Double>> proteinValuesConcentration, HashMap<String, HashMap<String, Double>> proteinValuesVolume, HashMap<String, Integer> proteinAccessionToCategory) {

        HashMap<Integer, ArrayList<Double>[]> result = new HashMap<Integer, ArrayList<Double>[]>();

        for (String accession : proteinValuesConcentration.keySet()) {
            Integer category = proteinAccessionToCategory.get(accession);
            if (category == null) {
                category = 3;
            }
            ArrayList<Double>[] categoryValues = result.get(category);
            if (categoryValues == null) {
                categoryValues = new ArrayList[2];
                categoryValues[0] = new ArrayList<Double>();
                categoryValues[1] = new ArrayList<Double>();
                result.put(category, categoryValues);
            }
            ArrayList<Double> means = categoryValues[0];
            ArrayList<Double> ratios = categoryValues[1];

            HashMap<String, Double> patientValuesConcentration = proteinValuesConcentration.get(accession);
            HashMap<String, Double> patientValuesVolume = proteinValuesVolume.get(accession);

            if (patientValuesVolume != null) {
                for (String patient : patientValuesConcentration.keySet()) {
                    Double intensityConcentration = patientValuesConcentration.get(patient);
                    Double intensityVolume = patientValuesVolume.get(patient);
                    if (intensityVolume != null && intensityVolume != 0 && intensityConcentration != 0) {
                        double ratio = intensityVolume / intensityConcentration;
                        ratio = BasicMathFunctions.log(ratio, 2);
                        double mean = (intensityVolume + intensityConcentration) / 2;
                        ratios.add(ratio);
                        means.add(mean);
                    }
                }
            }
        }

        return result;
    }

    private void writeVariabilityPlot(File destinationFile, HashMap<Integer, ArrayList<Double>[]> controlVariability, HashMap<Integer, ArrayList<Double>[]> ondVariability,
            HashMap<Integer, ArrayList<Double>[]> sasVariability, HashMap<Integer, ArrayList<Double>[]> rrmsVariability) throws IOException {

        HashMap<Integer, Integer> sizes = new HashMap<Integer, Integer>();
        for (Integer category : controlVariability.keySet()) {
            Integer size = sizes.get(category);
            if (size == null) {
                size = 0;
            }
            size += controlVariability.get(category)[0].size();
            sizes.put(category, size);
        }
        for (Integer category : ondVariability.keySet()) {
            Integer size = sizes.get(category);
            if (size == null) {
                size = 0;
            }
            size += ondVariability.get(category)[0].size();
            sizes.put(category, size);
        }
        for (Integer category : sasVariability.keySet()) {
            Integer size = sizes.get(category);
            if (size == null) {
                size = 0;
            }
            size += sasVariability.get(category)[0].size();
            sizes.put(category, size);
        }
        for (Integer category : rrmsVariability.keySet()) {
            Integer size = sizes.get(category);
            if (size == null) {
                size = 0;
            }
            size += rrmsVariability.get(category)[0].size();
            sizes.put(category, size);
        }

        int maxSize = 0;
        for (Integer size : sizes.values()) {
            if (size > maxSize) {
                maxSize = size;
            }
        }

        BufferedWriter bw = new BufferedWriter(new FileWriter(destinationFile));
        try {

            bw.write("Unaffected" + separator + separator + separator + "Uncertain" + separator + separator + separator + "Affected" + separator + separator + separator + "Other");
            bw.newLine();
            bw.write("Patient Group" + separator + "Intensity" + separator + "Variability" + separator + "Patient Group" + separator + "Intensity" + separator + "Variability" + separator + "Patient Group" + separator + "Intensity" + separator + "Variability" + separator + "Patient Group" + separator + "Intensity" + separator + "Variability");
            bw.newLine();

            int[] indexes = {0, 0, 0, 0};
            for (int i = 0; i < maxSize; i++) {
                for (int category = 0; category < indexes.length; category++) {
                    int index = i;
                    ArrayList<Double> intensities = controlVariability.get(category)[0];
                    ArrayList<Double> variabilities = controlVariability.get(category)[1];
                    if (index < intensities.size()) {
                        bw.write("Control" + separator + intensities.get(index) + separator + variabilities.get(index));
                        if (category < indexes.length - 1) {
                            bw.write(separator);
                        } else {
                            bw.newLine();
                        }
                    } else {
                        index -= intensities.size();
                        intensities = ondVariability.get(category)[0];
                        variabilities = ondVariability.get(category)[1];
                        if (index < intensities.size()) {
                            bw.write("OND" + separator + intensities.get(index) + separator + variabilities.get(index));
                            if (category < indexes.length - 1) {
                                bw.write(separator);
                            } else {
                                bw.newLine();
                            }
                        } else {
                            index -= intensities.size();
                            intensities = sasVariability.get(category)[0];
                            variabilities = sasVariability.get(category)[1];
                            if (index < intensities.size()) {
                                bw.write("SAS" + separator + intensities.get(index) + separator + variabilities.get(index));
                                if (category < indexes.length - 1) {
                                    bw.write(separator);
                                } else {
                                    bw.newLine();
                                }
                            } else {
                                index -= intensities.size();
                                intensities = rrmsVariability.get(category)[0];
                                variabilities = rrmsVariability.get(category)[1];
                                if (index < intensities.size()) {
                                    bw.write("RRMS" + separator + intensities.get(index) + separator + variabilities.get(index));
                                    if (category < indexes.length - 1) {
                                        bw.write(separator);
                                    } else {
                                        bw.newLine();
                                    }
                                } else {
                                    bw.write(separator + separator);
                                    if (category < indexes.length - 1) {
                                        bw.write(separator);
                                    } else {
                                        bw.newLine();
                                    }
                                }
                            }
                        }
                    }
                }
            }

        } finally {
            bw.close();
        }
    }

    private HashMap<Integer, ArrayList<Double>[]> getVariability(HashMap<String, HashMap<String, Double>> proteinValues, HashMap<String, Integer> proteinAccessionToCategory) {

        HashMap<Integer, ArrayList<Double>[]> result = new HashMap<Integer, ArrayList<Double>[]>();

        for (String accession : proteinValues.keySet()) {
            Integer category = proteinAccessionToCategory.get(accession);
            if (category == null) {
                category = 3;
            }
            ArrayList<Double>[] categoryIntensities = result.get(category);
            if (categoryIntensities == null) {
                categoryIntensities = new ArrayList[2];
                categoryIntensities[0] = new ArrayList<Double>();
                categoryIntensities[1] = new ArrayList<Double>();
                result.put(category, categoryIntensities);
            }
            ArrayList<Double> intensities = categoryIntensities[0];
            ArrayList<Double> variabilities = categoryIntensities[1];

            HashMap<String, Double> patientValues = proteinValues.get(accession);
            ArrayList<Double> values = new ArrayList<Double>(patientValues.values());
            double median = BasicMathFunctions.mean(values);
            if (median == 0) {
                throw new IllegalArgumentException("Null intensity");
            }
            double mad = BasicMathFunctions.std(values);
            double variability = mad / median;
            intensities.add(mad);
            variabilities.add(variability);
        }

        return result;
    }

    private HashMap<String, HashMap<String, Double>> normalizeAll(HashMap<String, HashMap<String, Double>> input, HashMap<String, Double> normalizationFactors) {
        HashMap<String, HashMap<String, Double>> result = new HashMap<String, HashMap<String, Double>>(input.size());
        for (String accession : input.keySet()) {
            HashMap<String, Double> accessionInput = input.get(accession);
            HashMap<String, Double> accessionNormalized = result.get(accession);
            if (accessionNormalized == null) {
                accessionNormalized = new HashMap<String, Double>();
                result.put(accession, accessionNormalized);
            }
            for (String patient : accessionInput.keySet()) {
                Double normalizationFactor = normalizationFactors.get(patient);
                if (normalizationFactor == null) {
                    throw new IllegalArgumentException("No normalization factor for patient " + patient + ".");
                }
                Double value = accessionInput.get(patient);
                Double normalizedValue = value / normalizationFactor;
                accessionNormalized.put(patient, normalizedValue);
            }
        }
        return result;
    }

    private HashMap<String, Double> getMeanAll(HashMap<String, HashMap<String, Double>> rawIntensities) {
        HashMap<String, ArrayList<Double>> patientMap = new HashMap<String, ArrayList<Double>>();
        for (String accession : rawIntensities.keySet()) {
            HashMap<String, Double> accessionMap = rawIntensities.get(accession);
            for (String patient : accessionMap.keySet()) {
                ArrayList<Double> patientIntensities = patientMap.get(patient);
                if (patientIntensities == null) {
                    patientIntensities = new ArrayList<Double>();
                    patientMap.put(patient, patientIntensities);
                }
                patientIntensities.add(accessionMap.get(patient));
            }
        }
        HashMap<String, Double> result = new HashMap<String, Double>();
        for (String patient : patientMap.keySet()) {
            ArrayList<Double> intensities = patientMap.get(patient);
            result.put(patient, BasicMathFunctions.mean(intensities));
        }
        return result;
    }

    private HashMap<String, Double> getMedianAll(HashMap<String, HashMap<String, Double>> rawIntensities) {
        HashMap<String, ArrayList<Double>> patientMap = new HashMap<String, ArrayList<Double>>();
        for (String accession : rawIntensities.keySet()) {
            HashMap<String, Double> accessionMap = rawIntensities.get(accession);
            for (String patient : accessionMap.keySet()) {
                ArrayList<Double> patientIntensities = patientMap.get(patient);
                if (patientIntensities == null) {
                    patientIntensities = new ArrayList<Double>();
                    patientMap.put(patient, patientIntensities);
                }
                patientIntensities.add(accessionMap.get(patient));
            }
        }
        HashMap<String, Double> result = new HashMap<String, Double>();
        for (String patient : patientMap.keySet()) {
            ArrayList<Double> intensities = patientMap.get(patient);
            result.put(patient, BasicMathFunctions.median(intensities));
        }
        return result;
    }

    private HashMap<String, Double> getMedianBrain(HashMap<String, HashMap<String, Double>> rawIntensities, HashMap<String, Integer> proteinAccessionToCategory) {
        HashMap<String, ArrayList<Double>> patientMap = new HashMap<String, ArrayList<Double>>();
        for (String accession : rawIntensities.keySet()) {
            Integer category = proteinAccessionToCategory.get(accession);
            if (category != null && category == 0) {
                HashMap<String, Double> accessionMap = rawIntensities.get(accession);
                for (String patient : accessionMap.keySet()) {
                    ArrayList<Double> patientIntensities = patientMap.get(patient);
                    if (patientIntensities == null) {
                        patientIntensities = new ArrayList<Double>();
                        patientMap.put(patient, patientIntensities);
                    }
                    patientIntensities.add(accessionMap.get(patient));
                }
            }
        }
        HashMap<String, Double> result = new HashMap<String, Double>();
        for (String patient : patientMap.keySet()) {
            ArrayList<Double> intensities = patientMap.get(patient);
            result.put(patient, BasicMathFunctions.median(intensities));
        }
        return result;
    }

    private void writeNormalizedInputForPCA(BufferedWriter bw, HashMap<String, Integer> proteinAccessionToCategory, HashMap<String, ArrayList<Double>> input) throws IOException {

        for (String accession : input.keySet()) {
            ArrayList<Double> intensities = input.get(accession);
            if (intensities.size() > 3) {
                Integer category = proteinAccessionToCategory.get(accession);
                if (category == null) {
                    category = 4;
                }
                double median = BasicMathFunctions.median(intensities);
                if (median > 0) {
                    ArrayList<Double> normalizedIntensities = new ArrayList<Double>();
                    for (double intensity : intensities) {
                        double normalizedIntensity = intensity / median;
                        normalizedIntensities.add(normalizedIntensity);
                    }
                    bw.write(category + "");
                    for (double intensity : normalizedIntensities) {
                        bw.write(separator + intensity);
                    }
                    bw.newLine();
                }
            }
        }
    }

    /**
     * 0 -> unaffected 1 -> uncertain 2 -> affected null -> other
     *
     * @param inputUnaffected
     * @param inputUncertain
     * @param inputAffected
     * @return
     */
    private HashMap<String, Integer> loadProteinMapping(File inputUnaffected, File inputUncertain, File inputAffected) throws IOException {
        HashMap<String, Integer> result = new HashMap<String, Integer>();

        BufferedReader br = new BufferedReader(new FileReader(inputUnaffected));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 0);
            }
        } finally {
            br.close();
        }

        br = new BufferedReader(new FileReader(inputUncertain));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 1);
            }
        } finally {
            br.close();
        }

        br = new BufferedReader(new FileReader(inputAffected));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String name = line.trim();
                result.put(name, 2);
            }
        } finally {
            br.close();
        }

        return result;
    }

    private void loadPatientMapping(File inputFile, HashMap<String, String> nameToCategory, HashMap<String, String> keyToCategory, HashMap<String, String> nameToKey) throws IOException {

        BufferedReader br = new BufferedReader(new FileReader(inputFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                String[] content = line.split("\t");
                String name = content[0];
                String key = content[1];
                String category = content[2];
                nameToCategory.put(name, category);
                keyToCategory.put(key, category);
                nameToKey.put(name, key);
            }
        } finally {
            br.close();
        }
    }

    private void jillLoadIntensities(File inputFile, HashMap<String, String> categoryMapping, HashMap<String, String> nameToKey, HashMap<String, HashMap<String, Double>> controlMap, HashMap<String, HashMap<String, Double>> sasMap, HashMap<String, HashMap<String, Double>> ondMap, HashMap<String, HashMap<String, Double>> rrmsMap) throws FileNotFoundException, IOException {

        BufferedReader br = new BufferedReader(new FileReader(inputFile));
        try {

            String line = br.readLine();
            ArrayList<String> names = new ArrayList<String>();
            String[] content = line.split("\t");
            for (int i = 5; i < 106; i++) {
                names.add(content[i]);
            }

            while ((line = br.readLine()) != null) {
                content = line.split("\t");
                String accession = content[0];
                for (int i = 5; i < 106; i++) {
                    String name = names.get(i - 5);
                    String category = categoryMapping.get(name);
                    if (category == null) {
                        throw new IllegalArgumentException("Name not recognized " + name + ".");
                    }
                    String key = name;
                    if (nameToKey != null) {
                        key = nameToKey.get(name);
                    }
                    String component = "";
                    if (content.length > i) {
                        component = content[i].trim();
                    }
                    if (!component.equals("")) {
                        Double value;
                        try {
                            value = new Double(content[i]);
                        } catch (Exception e) {
                            throw new IllegalArgumentException("Failed to convert " + content[i] + ", " + i + " of accession " + accession);
                        }
                        if (category.equals("SAS")) {
                            HashMap<String, Double> sasIntensities = sasMap.get(accession);
                            if (sasIntensities == null) {
                                sasIntensities = new HashMap<String, Double>();
                                sasMap.put(accession, sasIntensities);
                            }
                            sasIntensities.put(key, value);

                            HashMap<String, Double> controlIntensities = controlMap.get(accession);
                            if (controlIntensities == null) {
                                controlIntensities = new HashMap<String, Double>();
                                controlMap.put(accession, controlIntensities);
                            }
                            controlIntensities.put(key, value);
                        } else if (category.equals("OND")) {
                            HashMap<String, Double> ondIntensities = ondMap.get(accession);
                            if (ondIntensities == null) {
                                ondIntensities = new HashMap<String, Double>();
                                ondMap.put(accession, ondIntensities);
                            }
                            ondIntensities.put(key, value);

                            HashMap<String, Double> controlIntensities = controlMap.get(accession);
                            if (controlIntensities == null) {
                                controlIntensities = new HashMap<String, Double>();
                                controlMap.put(accession, controlIntensities);
                            }
                            controlIntensities.put(key, value);
                        } else if (category.equals("RRMS")) {
                            HashMap<String, Double> rrmsIntensities = rrmsMap.get(accession);
                            if (rrmsIntensities == null) {
                                rrmsIntensities = new HashMap<String, Double>();
                                rrmsMap.put(accession, rrmsIntensities);
                            }
                            rrmsIntensities.put(key, value);
                        } else {
                            throw new IllegalArgumentException("Name not recognized " + name + ".");
                        }
                    }
                }
            }

        } finally {
            br.close();
        }
    }

}
