package com.compomics.scripts_marc;

import com.compomics.util.Util;
import com.compomics.util.experiment.identification.protein_sequences.FastaIndex;
import com.compomics.util.experiment.identification.protein_sequences.SequenceFactory;
import com.compomics.util.protein.Header;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

/**
 *
 * @author Marc Vaudel
 */
public class LeishmaniaData {

    private SequenceFactory sequenceFactory;

    private static final String separator = "\t";

    public static void main(String[] args) {

        LeishmaniaData m = new LeishmaniaData();

        try {
            m.separateSpecies();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void separateSpecies() throws IOException, FileNotFoundException, ClassNotFoundException, InterruptedException {

        sequenceFactory = SequenceFactory.getInstance(1000000);
        sequenceFactory.loadFastaFile(new File("D:\\projects\\Probe Service\\Leishmania\\databases\\uniprot-Leishmania_Major-Mus_Musculus-contaminants-trypsin-27.05.15_concatenated_target_decoy.fasta"));

        File fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\extracellular\\proteins1.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\extracellular\\proteins2.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\extracellular\\proteins3.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\intracellular\\proteins1.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\intracellular\\proteins2.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\intracellular\\proteins3.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\mouse\\proteins1.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\mouse\\proteins2.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\mouse\\proteins3.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\intracellular\\proteins.txt");
        separateFile(fileIn);
        fileIn = new File("D:\\projects\\Probe Service\\Leishmania\\extracellular\\proteins.txt");
        separateFile(fileIn);
    }

    public void separateFile(File fileIn) throws FileNotFoundException, IOException, InterruptedException, ClassNotFoundException {
        String filePath = fileIn.getParent();
        String fileName = Util.removeExtension(fileIn.getName());
        File leishmania = new File(filePath, fileName + "_Leishmania.txt");
        File mouse = new File(filePath, fileName + "_Mouse.txt");
        File mouse2 = new File(filePath, fileName + "_Mouse2.txt");
        File mix = new File(filePath, fileName + "_Mix.txt");
        File contaminants = new File(filePath, fileName + "_Contaminants.txt");

        BufferedReader br = new BufferedReader(new FileReader(fileIn));
        BufferedWriter bwLeishmania = new BufferedWriter(new FileWriter(leishmania));
        BufferedWriter bwMouse = new BufferedWriter(new FileWriter(mouse));
        BufferedWriter bwMouse2 = new BufferedWriter(new FileWriter(mouse2));
        BufferedWriter bwMix = new BufferedWriter(new FileWriter(mix));
        BufferedWriter bwContaminants = new BufferedWriter(new FileWriter(contaminants));

        String line = br.readLine();
        bwLeishmania.write(line);
        bwLeishmania.newLine();
        bwMouse.write(line);
        bwMouse.newLine();
        bwMouse2.write(line);
        bwMouse2.newLine();
        bwMix.write(line);
        bwMix.newLine();
        bwContaminants.write(line);
        bwContaminants.newLine();

        while ((line = br.readLine()) != null) {

            String[] splitLine = line.split(separator);
            String[] accessions = splitLine[15].split(", ");
            SpeciesCategory speciesCategory = getSpeciesCategory(accessions);
            if (speciesCategory == SpeciesCategory.contaminant) {
                bwContaminants.write(line);
                bwContaminants.newLine();
            } else if (speciesCategory == SpeciesCategory.mix) {
                bwMix.write(line);
                bwMix.newLine();
            } else if (speciesCategory == SpeciesCategory.leishmania) {
                bwLeishmania.write(line);
                bwLeishmania.newLine();
            } else if (speciesCategory == SpeciesCategory.mouse) {
                bwMouse.write(line);
                bwMouse.newLine();
            } else if (speciesCategory == SpeciesCategory.otherMouse) {
                bwMouse2.write(line);
                bwMouse2.newLine();
            } else {
                throw new UnsupportedOperationException("Species not supported");
            }

        }

        br.close();
        bwLeishmania.close();
        bwMouse.close();
        bwMouse2.close();
        bwMix.close();
        bwContaminants.close();
    }

    private SpeciesCategory getSpeciesCategory(String[] accessions) throws IOException, InterruptedException, ClassNotFoundException {
        boolean leishmania = false;
        boolean mouse = false;
        boolean otherMouse = false;
        for (String accession : accessions) {
            Header header = sequenceFactory.getHeader(accession);
            if (header.getTaxonomy() == null) {
                return SpeciesCategory.contaminant;
            }
            String taxonomy = header.getTaxonomy().toLowerCase();
            if (taxonomy == null || taxonomy.length() == 0 || taxonomy.contains("sus scrofa") || taxonomy.contains("homo")) {
                return SpeciesCategory.contaminant;
            } else if (taxonomy.contains("leishmania")) {
                leishmania = true;
            } else if (taxonomy.contains("mus") || taxonomy.contains("rattus")) {
                mouse = true;
            } else if ( taxonomy.contains("virus") || taxonomy.contains("rattus") || taxonomy.contains("intracisternal") || taxonomy.contains("cryptosporidium")) {
                otherMouse = true;
            } else {
                throw new UnsupportedOperationException("Species not supported");
            }
        }
            if (leishmania && mouse) {
                return SpeciesCategory.mix;
            } else if (leishmania) {
                return SpeciesCategory.leishmania;
            } else if (mouse) {
                return SpeciesCategory.mouse;
            } else if (otherMouse) {
                return SpeciesCategory.otherMouse;
            }
            throw new UnsupportedOperationException("Species not supported");
    }
    

    private enum SpeciesCategory {

        mouse, otherMouse, leishmania, mix, contaminant;
    }

}
