package com.compomics.marc;

import com.compomics.util.exceptions.ExceptionHandler;
import com.compomics.util.exceptions.exception_handlers.CommandLineExceptionHandler;
import com.compomics.util.experiment.biology.Peptide;
import com.compomics.util.experiment.identification.spectrum_assumptions.PeptideAssumption;
import com.compomics.util.experiment.identification.protein_sequences.SequenceFactory;
import com.compomics.util.experiment.identification.SpectrumIdentificationAssumption;
import com.compomics.util.experiment.identification.matches.ModificationMatch;
import com.compomics.util.experiment.identification.matches.SpectrumMatch;
import com.compomics.util.experiment.io.identifications.idfilereaders.PepxmlIdfileReader;
import com.compomics.util.experiment.massspectrometry.Spectrum;
import com.compomics.util.io.export.styles.DefaultStyle;
import com.compomics.util.io.export.writers.ExcelWriter;
import com.compomics.util.preferences.SequenceMatchingPreferences;
import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.xml.bind.JAXBException;
import org.xmlpull.v1.XmlPullParserException;

/**
 *
 * @author Marc
 */
public class PepxmlFileReaderTest {

    /**
     * Basic exception handler
     */
    private ExceptionHandler exceptionHandler = new CommandLineExceptionHandler();

    public static void main(String[] args) {
        PepxmlFileReaderTest m = new PepxmlFileReaderTest();

        try {
            m.testParsing();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testParsing() throws IOException, SQLException, ClassNotFoundException, InterruptedException, JAXBException, IllegalArgumentException, XmlPullParserException {

        File fastaFile = new File("D:\\databases\\uniprot-human-reviewed-trypsin-august-2014_concatenated_target_decoy.fasta");
        SequenceFactory sequenceFactory = SequenceFactory.getInstance(1000000);
        sequenceFactory.loadFastaFile(fastaFile, null);
        sequenceFactory.getDefaultProteinTree(null, exceptionHandler);
        SequenceMatchingPreferences sequenceMatchingPreferences = new SequenceMatchingPreferences();
        sequenceMatchingPreferences.setSequenceMatchingType(SequenceMatchingPreferences.MatchingType.indistiguishableAminoAcids);
        sequenceMatchingPreferences.setLimitX(0.25);

        File inputFile = new File("D:\\projects\\PeptideShaker\\test files\\2 identification files\\qExactive01819.comet.pep.xml");

        PepxmlIdfileReader pepxmlIdfileReader = new PepxmlIdfileReader(inputFile);

        LinkedList<SpectrumMatch> spectrumMatches = pepxmlIdfileReader.getAllSpectrumMatches(null, null);

        File outputFile = new File("C:\\Users\\mva037\\Desktop\\test\\cometTest.xls");

        ExcelWriter ew = new ExcelWriter(outputFile);
        DefaultStyle style = DefaultStyle.getStyle(ew);
        ew.setWorkbookStyle(style);

        ew.startNewSection("ids");
        ew.addSeparator();
        ew.writeHeaderText("Spectrum Title");
        ew.addSeparator();
        ew.writeHeaderText("Peptide");
        ew.addSeparator();
        ew.writeHeaderText("Modifications");
        ew.addSeparator();
        ew.writeHeaderText("Score");
        ew.addSeparator();
        ew.writeHeaderText("decoy");
        ew.newLine();

        int cpt = 0;
        for (SpectrumMatch spectrumMatch : spectrumMatches) {
            ew.write(cpt++ + "");
            ew.addSeparator();
            ew.write(Spectrum.getSpectrumTitle(spectrumMatch.getKey()));
            ew.addSeparator();
            PeptideAssumption bestAssumption = null;
            Double bestEvalue = null;
            for (SpectrumIdentificationAssumption assumption : spectrumMatch.getAllAssumptions()) {
                PeptideAssumption peptideAssumption = (PeptideAssumption) assumption;
                double eValue = peptideAssumption.getScore();
                if (bestEvalue == null || bestEvalue > eValue) {
                    bestAssumption = peptideAssumption;
                    bestEvalue = eValue;
                }
            }
            Peptide peptide = bestAssumption.getPeptide();
            ew.write(peptide.getSequence());
            ew.addSeparator();
            if (peptide.isModified()) {
                for (ModificationMatch modificationMatch : peptide.getModificationMatches()) {
                    ew.write(modificationMatch.getTheoreticPtm() + "(" + modificationMatch.getModificationSite() + ")");
                }
            }
            ew.addSeparator();
            ew.write(bestAssumption.getScore() + "");
            ew.addSeparator();
            ArrayList<String> proteins = peptide.getParentProteins(sequenceMatchingPreferences);
            boolean decoy = false;
            boolean target = false;
            for (String protein : proteins) {
                if (sequenceFactory.isDecoyAccession(protein)) {
                    decoy = true;
                } else {
                    target = true;
                }
            }
            if (decoy && !target) {
                ew.write("1");
            } else if (target && !decoy) {
                ew.write("0");
            }
            ew.addSeparator();
            ew.newLine();
        }
        ew.close();
        sequenceFactory.closeFile();
    }
}
